/*
 * Created on 17.09.2004
 *
 * (C) Peter Weigerber <peter.weissgerber@ku-eichstaett.de>
 * 
 * For license information see the LICENSE file that is shipped
 * with the source code.
 *
 * 
 */
package org.eposoft.epos.eposee.pixmap.data;

/** Binary coupling class. 
 * Represents a (directed) coupling between two
 * items (only item id stored).
 * 
 * @author Peter Weigerber <peter.weissgerber@ku-eichstaett.de>
 * 
 */
public class BinaryCouplingI {
	private int from;
	private int to;
	private int support;
	private short confidence;
	
	/** Create new binary coupling on integers.
	 * 
	 * @param _from antecedent of the coupling
	 * @param _to consequent of the coupling
	 * @param _sup support value
	 * @param _conf confidence value
	 */
	public BinaryCouplingI(int _from, int _to, int _sup, double _conf) {
		from=_from;
		to=_to;
		support=_sup;
		confidence = (short)(_conf *1000);
	}
	
	/** Get ID of the antecedent of the coupling. 
	 * 
	 * @return ID of the antecedent
	 */
	public int getAntecedent() {
		return from;
	}
	
	/** Get ID of the consequent of the coupling.
	 * 
	 * @return ID of the consequent
	 */
	public int getConsequent() {
		return to;
	}
	
	/** Get support count of the coupling.
	 * 
	 * @return support count
	 */
	public int getSupportCount() {
		return support;
	}
	
	/** Get confidence of the coupling.
	 * 
	 * @return confidence
	 */
	public double getConfidence() {
		return ((double)confidence)/(double)1000;
	}
	
	/** Set new antecedent.
	 * 
	 * @param _from item ID
	 */
	protected void setAntecedent(int _from) {
		from = _from;
	}
	
	/** Set new consequent.
	 * 
	 * @param _to item ID
	 */
	protected void setConsequent(int _to) {
		to = _to;
	}
}
