/*
 * Created on 21.04.2004
 * 
 * (C) Peter Weigerber <peter.weissgerber@ku-eichstaett.de>
 * 
 * For license information see the LICENSE file that is shipped
 * with the source code.
 *
 */
package org.eposoft.epos.eposee.pixmap.gui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Shell;

/** This class implements a simple integer selection dialog.
 * 
 * @author Peter Weigerber <peter.weissgerber@ku-eichstaett.de>
 *
 */
public class IntegerSelectionDialog extends Dialog {

	private Button btnOkay;
	private Button btnCancel;
	private Scale scale;
	private Label selected;
	private int min = 0;
	private int max = 1;
	private int def = 0;
	private Integer response;

	/** Create a new integer selection dialog.
	 * 
	 * @param arg0 parent shell
	 * @param minimum minimum value
	 * @param maximum maximum value
	 * @param defaultval default preselected value
	 */
	public IntegerSelectionDialog(
		Shell arg0,
		int minimum,
		int maximum,
		int defaultval) {
		super(arg0);
		min = minimum;
		max = maximum;
		def = defaultval;
	}

	/** Open the dialog an let the user make a choice.
	 * 
	 * @return integer value selected by the user
	 */
	public Integer open() {
		response = null;
		final Shell shell =
			new Shell(getParent(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("Please select new value");
		shell.setSize(330, 125);
		scale = new Scale(shell, SWT.HORIZONTAL);
		scale.setLocation(25, 20);
		scale.setSize(250, 20);
		scale.setMinimum(min);
		scale.setMaximum(max);
		scale.setSelection(def);
		//scale.setPageIncrement(max);

		selected = new Label(shell, SWT.NONE);
		selected.setLocation(285, 20);
		selected.setSize(30, 20);
		selected.setText("" + scale.getSelection());

		Listener vchangelistener = new Listener() {
			public void handleEvent(Event event) {
				selected.setText("" + scale.getSelection());
			}
		};
		scale.addListener(SWT.Selection, vchangelistener);

		btnOkay = new Button(shell, SWT.PUSH);
		btnOkay.setText("Ok");
		btnOkay.setLocation(30, 60);
		btnOkay.setSize(75, 35);
		btnCancel = new Button(shell, SWT.PUSH);
		btnCancel.setText("Cancel");
		btnCancel.setLocation(195, 60);
		btnCancel.setSize(75, 35);
		Listener listener = new Listener() {
			public void handleEvent(Event event) {
				if (event.widget == btnOkay) {
					response = new Integer(scale.getSelection());
				}
				else {
					response = null;
				}
				shell.close();
			}
		};
		btnOkay.addListener(SWT.Selection, listener);
		btnCancel.addListener(SWT.Selection, listener);
		shell.open();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}

		return response;
	}

}
