/*
 * Created on 16.04.2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package org.eposoft.epos.eposee.pixmap.gui;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eposoft.epos.eposee.pixmap.data.BinaryCouplingI;
import org.eposoft.epos.eposee.pixmap.data.SparePixMap;

/**
 * @author Peter Weigerber <peter.weissgerber@ku-eichstaett.de>
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class PixMapCanvas extends Canvas {
	
	public static final int PIXMAP_SHOW_CONFIDENCE = 0;
	
	public static final int PIXMAP_SHOW_SUPPORT = 1;

	private int displayed_metric = PIXMAP_SHOW_CONFIDENCE;
	
	private SparePixMap couplingsmatrix;

	private int facx = 1;

	private int spacex = 0;

	private int startx = 1;

	private RGB[] colorscale1 = SWTColorScales.COLD_HOT;

	private Color background_color;
	
	private Color empty_color;

	private Image myimage;
	
	private int startrange = 0;
	
	private int rangelength = 0;
	
	private int maxrange = 4000;
	
	private int max_support = -1;
	
	private int normalize_support = -1;
	
	private int minsup = 1;
	
	private double minconf = 0.0;

	
	public PixMapCanvas(Composite arg0, int arg1) {
		super(arg0, arg1);
		Display display = this.getDisplay();
		background_color = new Color(display, 127, 127, 127);
		empty_color = new Color(display, 255, 255,255);
		this.addPaintListener(new MyPaintListener());
	}

	public int getSx() {
	    return 2 * startx + (facx + spacex) * rangelength;
	}
	
	private void paintImage() {
		
		int sx = 2 * startx + (facx + spacex) * rangelength;
		System.out.println("Size of the image: "+sx);
		if (myimage!=null) { 
			myimage.dispose();
		}
		myimage = new Image(getDisplay(), sx, sx);
		setBounds(0,0,sx,sx);

		GC gc = new GC(myimage);
		gc.setBackground(background_color);
		gc.fillRectangle(0, 0, sx, sx);
		
		gc.setBackground(empty_color);
		gc.fillRectangle(startx,startx, (facx+spacex)*rangelength, (facx+spacex)*rangelength);

		Color confcolor = new Color(getDisplay(), new RGB(0, 0, 0));

		BinaryCouplingI[] coups = couplingsmatrix.getCouplings();
		if ((displayed_metric == PIXMAP_SHOW_SUPPORT) && (max_support<0)) {
			// search maximum support
			computeMaxSupport();
			normalize_support=max_support;
			System.out.println("Normalizing to new support count: "+normalize_support);
		}
		int coupssize = coups.length;
		int lastpercent = 0;
		int tmp_posx;
		int tmp_posy;
		int tmp_posx_rg;
		int tmp_posy_rg;
		double tmp_cval;
		
		for (int i = 0; i < coupssize; i++) {
			int percent = (int)(((double)i/(double)coupssize) * 100);
			if (percent%10==0) {
				if (percent>lastpercent) {
					System.out.println(percent+" % of couplings painted.");
					lastpercent=percent;
				}
			}
			tmp_posx =coups[i].getAntecedent();
			tmp_posx_rg = tmp_posx - startrange;
			tmp_posy =coups[i].getConsequent();
			tmp_posy_rg = tmp_posy - startrange;
			tmp_cval = 0.0;
			switch (displayed_metric) {
				case PIXMAP_SHOW_CONFIDENCE:
					tmp_cval = coups[i].getConfidence();
					break;
				case PIXMAP_SHOW_SUPPORT:
					tmp_cval = ((double)coups[i].getSupportCount())/((double)normalize_support);
					if (tmp_cval>1) { tmp_cval = 1; }
					break;
			}
			
			confcolor = new Color(
					getDisplay(),
					colorscale1[(int) ((colorscale1.length - 1) * tmp_cval)]);
			gc.setBackground(confcolor);
			// only paint if point is in clipping area and sup>=minsup and conf>=minconf
			if ((tmp_posx_rg>=0) 
					&& (tmp_posx_rg<rangelength) 
					&& (tmp_posy_rg>=0) 
					&& (tmp_posy_rg<rangelength)
					&& (coups[i].getConfidence()>=minconf)
					&& (coups[i].getSupportCount()>=minsup)){
				gc.fillRectangle(startx + tmp_posx_rg * facx + tmp_posx_rg * spacex,
						startx + tmp_posy_rg * facx + tmp_posy_rg * spacex, facx, facx);
			}

			}

		confcolor.dispose();
		gc.dispose();

	}

	public class MyPaintListener implements PaintListener {

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.events.PaintListener#paintControl(org.eclipse.swt.events.PaintEvent)
		 */
		public void paintControl(PaintEvent arg0) {
			if (myimage != null) {
				GC gc = arg0.gc;
				gc.drawImage(myimage, 0, 0);

				int sx = 2 * startx + (facx + spacex) * rangelength;
				setSize(sx, sx);
			}

		}

	}

	/**
	 * Set color scale for confidence values
	 * 
	 * @param c
	 *                   Colors to use
	 */
	public void setConfidenceColorScale(RGB[] c) {
		colorscale1 = c;
		this.paintImage();
		this.redraw();
	}

	/**
	 * Set the background color
	 * 
	 * @param c
	 *                   new RGB color
	 */
	public void setBackgroundColor(RGB c) {
		background_color = new Color(getDisplay(), c);
		this.paintImage();
		this.redraw();
	}

	/**
	 * Get the background color
	 * 
	 * @return RGB background color
	 */
	public RGB getBackgroundColor() {
		return background_color.getRGB();
	}

	/**
	 * Sets the zoom factor for both x- and y-axis. Each item of a rule is
	 * displayed as rectangle of xzoom*yzoom pixels.
	 * 
	 * @param xzoom
	 *                   x-axis zoom
	 * @param yzoom
	 *                   y-axis zoom
	 */
	public void setZoom(int xzoom) {
		facx = xzoom;
		this.paintImage();
		this.redraw();
	}

	/**
	 * Get zoom
	 * 
	 * @return zoom factor
	 */
	public int getZoom() {
		return facx;
	}
	
	/** Set new color for items that are not in rule */
	public void setEmptyFieldColor(RGB c) {
		empty_color = new Color(getDisplay(), c);
		this.paintImage();
		this.redraw();
	}

	
	/** Set new pixmap for this view and paint it
	 * 
	 * @param _pixmap pixmap to paint
	 */
	public void setPixMap(SparePixMap _pixmap) {
		couplingsmatrix = _pixmap;
		if (couplingsmatrix.size()<maxrange) { // TODO do this as property
			startrange = 0;
			rangelength = couplingsmatrix.size();
		}
		else {
			startrange = 0;
			rangelength=maxrange;
		}
		max_support=-1;
		normalize_support=-1;
		this.paintImage();
		this.redraw();
	}
		
	/**
	 * Get the number of the item at a specific position of this
	 * panel.
	 * 
	 * @param xyvalue
	 *                   position
	 * @return number of the item or -1 if no item at the given position
	 */
	public int getItemNumber(int xyvalue) {
		xyvalue=xyvalue + startrange;
		int xoffset = xyvalue - startx;
		int xrow = (xoffset) / (spacex + facx );
		if ((xrow < 0) || (xrow >= couplingsmatrix.size())) {
			return -1;
		}
		return xrow;
	}

	/** Get the pixelmap that is shown in this canvas
	 * 
	 * @return pixmap of this canvas
	 */
	public SparePixMap getPixMap() {
		return couplingsmatrix;
	}
	
	public String[] getItems() {
		return couplingsmatrix.getMappings();
	}
	
	/** Sets new start of the clipping area in the pixelmap that should be shown
	 * 
	 * @param _start start of the clipping range
	 */
	public void setClippingAreaStart(int _start) {
		if (_start<=(couplingsmatrix.size()-rangelength)) {
			startrange = _start;
			this.paintImage();
			this.redraw();
		}
		else {
			System.out.println("Start value was too high");
		}
	}
	
	public int getClippingAreaStart() {
		return startrange;
	}
	
	/** Set metric to display 
	 * 
	 * @param _metric metric to show, use the PIXMAP_SHOW_... final integers
	 */
	public void setDisplayedMetric(int _metric) {
		displayed_metric=_metric;
		this.paintImage();
		this.redraw();
	}

	/** Get displayed metric
	 * 
	 * @return PIXMAP_SHOW_CONFIDENCE or PIXMAP_SHOW_SUPPORT
	 */
	public int getDisplayedMetric() {
		return displayed_metric;
	}
	
	/** Get normalize-to-support. 
	 * Support Values >=normalize-to-support are painted with the strongest color. 
	 * 
	 * @return max. support
	 */
	public int getNormalizeToSupportCount() {
		if (normalize_support<0) {
			computeMaxSupport();
			normalize_support = max_support;
		}
		return normalize_support;
	}
	
	/** Set normalize-to-support.
	 * Support Values >=normalize-to-support are painted with the strongest color.
	 * 
	 * @param _sup new normalize-to-support
	 */
	public void setNormalizeToSupportCount(int _sup) {
		normalize_support = _sup;
		this.paintImage();
		this.redraw();
	}
	
	/** Get highest support count in the pixelmap
	 * 
	 * @return maximal support count
	 */
	public int getMaxSupportCount() {
		if (max_support<0) {
			computeMaxSupport();
			normalize_support = max_support;
		}
		return max_support;
	}
	
	/** Set minimam support count
	 * 
	 * @param _minsup new minimal support count
	 */
	public void setMinSupportCount(int _minsup) {
		minsup = _minsup;
		this.paintImage();
		this.redraw();
	}
	
	/** Get minimal support count
	 * 
	 * @return minimal support count
	 */
	public int getMinSupportCount() {
		return minsup;
	}
	
	/** Set minimal confidence
	 * 
	 * @param _minconf new minimal confidence
	 */
	public void setMinConfidence(double _minconf) {
		minconf = _minconf;
		this.paintImage();
		this.redraw();
	}
	
	/** Get minimal confidence
	 * 
	 * @return minimal confidence
	 */
	public double getMinConfidence() {
		return minconf;
	}
	
	
	/** Set length of clipping area
	 * 
	 * @param _length
	 */
	public void setClippingAreaLength(int _length) {
		if (rangelength<=couplingsmatrix.size()) {
			rangelength=_length;
		}
		else {
			rangelength=couplingsmatrix.size();
			System.out.println("Clipping area length adjusted to size of pixmap.");
		}
		if ((startrange+rangelength)>=couplingsmatrix.size()) {
			startrange=couplingsmatrix.size()-rangelength-1;
			System.out.println("Clipping area start automatically adjusted.");
		}
		
		this.paintImage();
		this.redraw();
	}
	
	/** Get support count of a certain position of the displayed pixelmap
	 * 
	 * @param x position on x-axis
	 * @param y position on y-axis
	 * @return support count
	 */
	public int getSupportCount(int x, int y) {
		BinaryCouplingI cp = couplingsmatrix.getCouplingAt(x,y);
		if (cp==null) {
			return 0;
		}
		else {
			return cp.getSupportCount();
		}
	}
	
	/** Get confidence of a certain point in the pixelmap
	 * 
	 * @param x position at x-axis
	 * @param y position at y-axis
	 * @return confidence
	 */
	public double getConfidence(int x, int y) {
		BinaryCouplingI cp = couplingsmatrix.getCouplingAt(x,y);
		if (cp==null) {
			return 0.0;
		}
		else {
			return cp.getConfidence();
		}
	}
	
	public int getClippingAreaLength() {
		return rangelength;
	}
	
	private void computeMaxSupport() {
		BinaryCouplingI[] coups = couplingsmatrix.getCouplings();
		for (int i=0;i<coups.length;i++) {
			if (coups[i].getSupportCount()>max_support) {
				max_support=coups[i].getSupportCount();
				normalize_support = max_support;
			}
		}
	}
	
	/**
	 * Exports the visualization graphics (incl. zoom config etc.) into a PNG
	 * file.
	 * 
	 * @param filename
	 *                   Filename
	 * @throws IOException
	 *                   Thrown, if file cannot be written.
	 */
	public void exportToPNG(String filename) throws IOException {
		if (filename.toLowerCase().endsWith(".bmp")) {
			ImageLoader imageLoader = new ImageLoader();
			imageLoader.data = new ImageData[] { myimage.getImageData() };
			imageLoader.save(filename, SWT.IMAGE_BMP);
		}
		else if (filename.toLowerCase().endsWith(".png")) {
				ImageData imdata = myimage.getImageData();
				int width = imdata.width;
				int height = imdata.height;
				
				// We use AWT classes to construct an new image and save this
				// image with the JAVAX ImageIO, because SWT lacks
				// PNG export possibility at the moment
				
				BufferedImage image =
					new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
				
				for (int i=0;i<height;i++) {
					for (int j=0;j<width;j++) {
						image.setRGB(j,i,imdata.getPixel(j,i));
					}
				}
				ImageIO.write(image, "png", new File(filename));
		}
	}

}