/*
 * $Id: SetupRoseJob.java,v 1.9 2004/12/28 21:47:31 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.assist.mining;

import java.sql.SQLException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.softevo.rose.assist.AssistPlugin;
import org.softevo.rose.assist.mining.db.DatabaseOperations;
import org.softevo.rose.assist.mining.db.Tables;
import org.softevo.rose.data.entities.model.ChangedEntity;
import org.softevo.util.StringUtils;

/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class SetupRoseJob extends Job {

    public static final int WORK = 50;
    
    private DatabaseOperations dbOperations;

    /**
     * @param dbOperations
     */
    public SetupRoseJob(DatabaseOperations dbOperations) {
        super("Setup Rose tables");
        this.dbOperations = dbOperations;
    }

    
    /**
     * @see org.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.IProgressMonitor)
     */
    protected IStatus run(final IProgressMonitor monitor) {
        try {
            monitor.beginTask("Setup ROSE", WORK);
            monitor.subTask("Create lineitems table");
            insertLineitems(ChangedEntity.CHANGED);
            insertSpecialLineitems(ChangedEntity.ADDED, ChangedEntity.ADDED + "_IN");
            insertSpecialLineitems(ChangedEntity.DELETED, ChangedEntity.DELETED + "_IN");
            monitor.worked(30);
            
            monitor.subTask("Create activities table");
            String[] relevantActions = { ChangedEntity.ADDED, ChangedEntity.DELETED };
            insertActivities(relevantActions);
            monitor.worked(WORK - 30);
        } catch (SQLException e) {
            AssistPlugin.logException(e, Status.WARNING);
        } 
        if (monitor.isCanceled()) return Status.CANCEL_STATUS;
        return Status.OK_STATUS;
    }

        
    
    
	/**
	 * @throws SQLException
     */
    private void insertLineitems(String searchFor) throws SQLException {
		String query = "INSERT INTO " + dbOperations.getTableName(Tables.TABLE_LINEITEMS) + " "
			+ "SELECT t.TransaktionID, e.EntityID, Action, t.EndTime, t.IsNoise " 
			+ "FROM " 
				+ Tables.TABLE_TRANSACTIONS + " t, "
				+ Tables.TABLE_REVISIONS + " r, " 
				+ Tables.TABLE_CHANGED_ENTITIES + " e " 
			+ "WHERE "
				+ "e.RevisionID = r.RevisionID "
				+ "AND e.FileID = r.FileID "
				+ "AND r.TransaktionID = t.TransaktionID "
				+ "AND Action='" + searchFor + "';";
		dbOperations.executeUpdate(query);
    }

    /**
	 * @throws SQLException
     */
    private void insertSpecialLineitems(String searchFor, String insertAs) throws SQLException {
		String query = "INSERT INTO " + dbOperations.getTableName(Tables.TABLE_LINEITEMS) + " "
			+ "SELECT foo.TransaktionID, foo.FileID, '" + insertAs + "', foo.EndTime, foo.IsNoise FROM " 
			+ "(SELECT DISTINCT t.TransaktionID, r.FileID, t.EndTime, t.IsNoise " 
			+ "FROM " 
				+ Tables.TABLE_TRANSACTIONS + " t, "
				+ Tables.TABLE_REVISIONS + " r, " 
				+ Tables.TABLE_CHANGED_ENTITIES + " e " 
			+ "WHERE "
				+ "e.RevisionID = r.RevisionID "
				+ "AND e.FileID = r.FileID "
				+ "AND r.TransaktionID = t.TransaktionID "
				+ "AND Action='" + searchFor + "') foo;";
		dbOperations.executeUpdate(query);
    }


    /** 
	 * @throws SQLException
     */
    private void insertActivities(String[] actions) throws SQLException {
        // NOTE We consider CheckinTime instead of EndTime (and thus need no join with Transactions)
		String query = "INSERT INTO " + dbOperations.getTableName(Tables.TABLE_ACTIVITIES) + " "
			+ "SELECT e.EntityID, e.Action, r.CheckinTime, r.BranchPrefix " 
			+ "FROM " 
				+ Tables.TABLE_REVISIONS + " r, " 
				+ Tables.TABLE_CHANGED_ENTITIES + " e " 
			+ "WHERE "
				+ "e.RevisionID = r.RevisionID "
				+ "AND e.FileID = r.FileID "
				+ "AND Action IN " 
					+ "( '" + StringUtils.join(actions, "', '") + "' );";
		dbOperations.executeUpdate(query);
    }

}
