/*
 * $Id: PreprocessProjectAction.java,v 1.5.2.1 2005/04/09 12:43:48 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.assist.ui.actions;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.team.internal.ccvs.core.CVSException;
import org.eclipse.team.internal.ccvs.core.resources.CVSWorkspaceRoot;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.internal.Workbench;
import org.softevo.rose.assist.AssistPlugin;
import org.softevo.rose.assist.ui.wizards.PreprocessProjectWizard;

/** An action that pops up a wizard that allows to
 *  choose the database connection for a project. <BR>
 */
public class PreprocessProjectAction implements IObjectActionDelegate {
    
	/**
	 * Find the object associated with the given object when it is adapted to
	 * the provided class. Null is returned if the given object does not adapt
	 * to the given class
	 * 
	 * @param selection
	 * @param c
	 * @return Object
	 */
	public static Object getAdapter(Object adaptable, Class c) {
		if (c.isInstance(adaptable)) {
			return adaptable;
		}
		if (adaptable instanceof IAdaptable) {
			IAdaptable a = (IAdaptable) adaptable;
			Object adapter = a.getAdapter(c);
			if (c.isInstance(adapter)) {
				return adapter;
			}
		}
		return null;
	}
	
	/**
	 * Creates an array of the given class type containing all the
	 * objects in the selection that adapt to the given class.
	 * 
	 * @param selection
	 * @param c
	 * @return
	 */
	public static Object[] getSelectedAdaptables(ISelection selection, Class c) {
		ArrayList result = null;
		if (!selection.isEmpty()) {
			result = new ArrayList();
			Iterator elements = ((IStructuredSelection) selection).iterator();
			while (elements.hasNext()) {
				Object adapter = getAdapter(elements.next(), c);
				if (c.isInstance(adapter)) {
					result.add(adapter);
				}
			}
		}
		if (result != null && !result.isEmpty()) {
			return result.toArray((Object[])Array.newInstance(c, result.size()));
		}
		return (Object[])Array.newInstance(c, 0);
	}

    
    private IStructuredSelection selection;

    /** Runs the action and pops up the dialog. <BR>
     */
    public void run(IAction action) {
        IProject[] selectedProjects = selectedProjects = getSelectedProjects();
        //if (selectedProjects.length == 1) {
            PreprocessProjectWizard wizard = new PreprocessProjectWizard(selectedProjects);
            WizardDialog dialog = new WizardDialog(Workbench.getInstance().getActiveWorkbenchWindow().getShell(), wizard);
            dialog.open();
        //}
    }
    
	/**
	 * Returns the selected projects.
	 * 
	 * @return the selected projects
	 */
	protected IProject[] getSelectedProjects() {
		IResource[] selectedResources = getSelectedResources();
		if (selectedResources.length == 0) return new IProject[0];
		ArrayList projects = new ArrayList();
		for (int i = 0; i < selectedResources.length; i++) {
			IResource resource = selectedResources[i];
			if (resource.getType() == IResource.PROJECT) {
				projects.add(resource);
			}
		}
		return (IProject[]) projects.toArray(new IProject[projects.size()]);
	}
    
	/**
	 * Returns the selected resources.
	 * 
	 * @return the selected resources
	 */
	protected IResource[] getSelectedResources() {
		return (IResource[]) getSelectedResources(IResource.class);
	}
	
	/**
	 * Returns an array of the given class type c that contains all
	 * instances of c that are either contained in the selection or
	 * are adapted from objects contained in the selection.
	 * 
	 * @param c
	 * @return
	 */
	protected Object[] getSelectedResources(Class c) {
		return getSelectedAdaptables(selection, c);
	}

	/*
	 * Method declared on IActionDelegate.
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		if (selection instanceof IStructuredSelection) {
			this.selection = (IStructuredSelection) selection;
			if (action != null) {
				setActionEnablement(action);
			}
		}
	}
	
	/**
	 * Method invoked from <code>selectionChanged(IAction, ISelection)</code> 
	 * to set the enablement status of the action. The instance variable 
	 * <code>selection</code> will contain the latest selection so the methods
	 * <code>getSelectedResources()</code> and <code>getSelectedProjects()</code>
	 * will provide the proper objects.
	 * 
	 * This method can be overridden by subclasses but should not be invoked by them.
	 */
	protected void setActionEnablement(IAction action) {
	    action.setEnabled(isEnabled());
	}
	
	/**
     * @return
     */
    private boolean isEnabled() {
        if (AssistPlugin.getDefault().isPreprocessing()) return false;
        
        IProject[] projects = getSelectedProjects();
        if (projects.length<1) return false;

        for (int i = 0; i < projects.length; i++) {
            try {
                if (!CVSWorkspaceRoot.isSharedWithCVS(projects[i])) return false;
            } catch (CVSException e) {
                AssistPlugin.logException(e, Status.WARNING);
                return false;
            }
        }
        
        return true;
    }

    /*
	 * Method declared on IObjectActionDelegate.
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
//		if(targetPart != null) {
//			this.shell = targetPart.getSite().getShell();
//			this.targetPart = targetPart;
//		}
	}
    
}
