/*
 * $Id: IRepositoryProvider.java,v 1.5 2004/12/20 13:00:54 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.ccvs;

import java.io.File;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.softevo.rose.ccvs.listener.ILogOutputListener;
import org.softevo.rose.ccvs.listener.IRepositoryContentListener;
import org.softevo.rose.ccvs.parser.ILogParser;

/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public interface IRepositoryProvider {

	/**
	 * Checks out a revision of a file from the repository to a temporary file.  
	 * The file should be deleted on VM shutdown (<PRE>deleteOnExit()>/PRE>).
	 * 
	 * @param fileName  the file name to be checked out
	 * @param revision  the revision to be checked out
	 * @return  the location of the temporary File.
	 * @see File#deleteOnExit()
	 */
	public abstract IFile checkout(String fileName, String revision, IProgressMonitor monitor) throws RepositoryException;
	
	/**
	 * Performs a log operation for a single files. The log output is parsed with a 
	 * LogParser to get a LogInfo object. This LogInfo object is then processed with 
	 * a ILogOutputListener to perform the desired actions.
	 * 
	 * @param listener  the ILogOutputListener that processes a LogInfo object.
	 * @param parser  the ILogParser that creates a LogInfo object.
	 * @param fileName  the file for which logs should be parsed.
	 */
	public abstract void getLogsForFile(ILogOutputListener listener, ILogParser parser, String fileName, IProgressMonitor monitor) throws RepositoryException;
	
	/**
	 * Performs a log operation for all files. The log output of each single file is 
	 * parsed with a LogParser to get a LogInfo object. This LogInfo object is then 
	 * processed with a ILogOutputListener to perform the desired actions.
	 * 
	 * @param listener  the ILogOutputListener that processes a LogInfo object.
	 * @param parser  the ILogParser that creates a LogInfo object.
	 */
	public abstract void getLogs(ILogOutputListener listener, ILogParser parser, IProgressMonitor monitor) throws RepositoryException;
	
	/**
	 * Gets the contents of the repository that are in the workspace.
	 * 
	 * @param listener  the Listener that is called for each file and directory.
	 */
	public abstract void getRepositoryContent(IRepositoryContentListener listener, IProgressMonitor monitor) throws RepositoryException;
		
	public abstract void unshareResource(IResource resource, IProgressMonitor progress) throws RepositoryException;

	
	/**
	 * Opens the repository provider.
	 */
	public abstract void open() throws RepositoryException;
	
	/**
	 * Closes the repository provider.
	 */
	public abstract void close();
	
}
