/*
 * $Id: TagItem.java,v 1.2 2004/11/08 13:05:31 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.ccvs.model;

import java.io.Serializable;

/**
* This class represents a tag of a file in the log information.
 * <P>
 * It consists of a 
 * <UL>
 * <LI> <EM>tagName</EM>, e.g., <TT>v_382</TT>
 * <LI> <EM>revision</EM>, e.g., <TT>1.15</TT>
 * </UL>
 * The affected file is directly stored in the LogInfo object.
 * 
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class TagItem implements Serializable {

    /** The serialization ID */
	private static final long serialVersionUID = 3900981796546623025L;
	
	/** The name of the tag */
	private String tagName = null;
	/** The revision of this tag */
	private String revision = null;
	
	/**
	 * Creates a tag object.
	 * 
	 * @param tagName  the name of the tag.
	 * @param revision  the revision of the tag.
	 */
	public TagItem(String tagName, String revision) {
		this.tagName = tagName;
		this.revision = revision;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "Tag " + tagName + "  [revision = " + revision + "]";
	}

    /**
     * @return Returns the revision.
     */
    public String getRevision() {
        return revision;
    }

    /**
     * @return Returns the tagName.
     */
    public String getTagName() {
        return tagName;
    }
    
    /**
     * @param revision The revision to set.
     */
    public void setRevision(String revision) {
        this.revision = revision;
    }
    
    /**
     * @param tagName The tagName to set.
     */
    public void setTagName(String tagName) {
        this.tagName = tagName;
    }
}
