/*
 * $Id: ConnectionElement.java,v 1.5.2.1 2005/04/09 12:43:48 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.db.preferences;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.team.internal.ccvs.ui.CVSUIPlugin;
import org.eclipse.team.internal.ccvs.ui.ICVSUIConstants;
import org.eclipse.ui.model.IWorkbenchAdapter;
import org.softevo.rose.db.DBConnectionProperties;
import org.softevo.rose.db.InternalDBConnectionProperties;


/** Model object that encapsulates a database connection
 *  properties object. <BR>
 */
public class ConnectionElement implements IWorkbenchAdapter, IAdaptable {
  
  /** The properties wrapped. <BR>
   */
  private DBConnectionProperties properties = null;

  /** The project the properties are associated with. <BR>
   */
  private IProject project = null;
  
  /** Creates a new connection element. <BR>
   *  @param properties the database properties
   */
  public ConnectionElement(DBConnectionProperties properties) {
    this.properties = properties;
  }

  /** Creates a new connection element. <BR>
   *  @param properties the database properties
   */
  public ConnectionElement(DBConnectionProperties properties, IProject project) {
    this.properties = properties;
    this.project = project;
  }
  
  /** This element has no children. <BR>
   */
  public Object[] getChildren(Object o) {
    return new Object[0];
  }
  
  /** Uses the cvs repository image. <BR>
   */
  public ImageDescriptor getImageDescriptor(Object object) {
    return CVSUIPlugin.getPlugin().getImageDescriptor(ICVSUIConstants.IMG_REPOSITORY);
  }
  
  /** Uses the name of the connection properties. <BR>
   */
  public String getLabel(Object o) {
      if (properties instanceof InternalDBConnectionProperties) {
          if (project!=null) {
              return properties.getName() + " (for project " + project.getName() + ")";
          }
          return properties.getName();
      }
      return properties.getName() + " (" + properties.getDatabaseURL() + ")";
  }
  
  /** No parent is returned. <BR>
   */
  public Object getParent(Object o) {
    return null;
  }
  
  /** Returns this class. <BR>
   */
  public Object getAdapter(Class adapter) {
    if (adapter == IWorkbenchAdapter.class) {
      return this;
    } else {
      return null;
    }
  }
  
  /** Gets the properties element. <BR>
   */
  public DBConnectionProperties getProperties() {
    return properties;
  }
  
  /** Compares two elements by the name of their
   *  properties object. <BR>
   */
  public boolean equals(Object object) {
    ConnectionElement element;
    
    if ((object != null) && (object instanceof ConnectionElement)) {
      element = (ConnectionElement) object;
      return element.properties.getName().equals(properties.getName());
    } else {
      return false;
    }
  }
}
