/*
 * $Id: EditDBConnectionWizard.java,v 1.6 2005/01/12 16:06:13 dallmeier Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.db.wizards;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.internal.Workbench;
import org.softevo.rose.db.DBConnectionProperties;
import org.softevo.rose.db.DbPlugin;

/** A wizard that is used to edit a connection. <BR>
 */
public class EditDBConnectionWizard extends Wizard {

  /** The page that displays the editor. <BR>
   */
  private EditDBConnectionWizardPage editPage = null;
  
  /** The connection properties. <BR>
   */
  private DBConnectionProperties properties = null;
  
  /** The title for the wizard. <BR>
   */
  private String title = null;

  /** The properties that are set AFTER editing. <BR>
   */
  private DBConnectionProperties settings;
  
  /** Creates a new wizard instance. <BR>
   *  @param properties the properties object that is edited.
   *  @param title the title for the wizard
   */
  public EditDBConnectionWizard(DBConnectionProperties properties, String title) {
    super();
    this.properties = properties;
    this.title = title;
    setWindowTitle(title);
  }
  
  /** Adds the pages to the wizard. <BR>
   */
  public void addPages() {
    setNeedsProgressMonitor(true);
    editPage = new EditDBConnectionWizardPage("EditLocation", title, 
            DbPlugin.getDefault().getImageRegistry().getDescriptor("newlocation"), 
            properties);
    addPage(editPage);
  }
  
  /** Gets the current properties from the editor. <BR>
   */
  public DBConnectionProperties getEditedProperties() {
    return settings;
  }
  
  /** Changes the settings as requested or
   *  adds the new connection to the database. <BR>
   */
  public boolean performFinish() {
      
      settings = editPage.getEditor().getCurrentSettings();
      if (properties != null) {
          DbPlugin.getDefault().changeConnectionProperties(settings);
          return true;
      }
      try {
          DbPlugin.getDefault().addConnectionProperties(settings.getName(), settings.getDatabaseProperties());
          DbPlugin.getDefault().changeConnectionProperties(settings);
          return true;
      } catch (Exception exception) {
          ErrorDialog errorDialog = new ErrorDialog(Workbench.getInstance().getActiveWorkbenchWindow().getShell(),
                  "Failed to add the new location", 
                  "An exception occured during adding the new location. "  +
                  exception.getClass().getName() + ":" + exception.getMessage(),
                  Status.CANCEL_STATUS, IStatus.CANCEL);
          errorDialog.open();
          return true;
      }
  }
  
    public boolean canFinish() {
        return editPage.canFlipToNextPage();
    }
}
