/*
 * $Id: URLUtils.java,v 1.3 2005/03/01 12:55:25 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

/**
 * @author zimmerth
 */
public class URLUtils {


	/**
	 * Copies an internet resource.
	 * 
	 * @param url path to the original resource.
	 * @param file name of the destination file.
	 * @throws IOException if any file system error occurs.
	 */
	public static void copyResourceToFile(
		URL url,
		File file,
		final boolean follow)
			throws IOException {

			class URLConnectionThread extends Thread {

				private URL url;
				private StringBuffer buf;
				private boolean finished = false;

				public URLConnectionThread(URL url, StringBuffer buf) {
					this.url = url;
					this.buf = buf;
				}

				public void run() {
	
					HttpURLConnection con = null;
					
					try {
						finished = false;

						con = (HttpURLConnection) url.openConnection();
						con.setInstanceFollowRedirects(follow);
						con.connect();
						BufferedReader data = new BufferedReader
							(new InputStreamReader(con.getInputStream()));
		
						String separator = System.getProperty("line.separator");
						String line;
		
						while ((line = data.readLine()) != null) {
							buf.append(line + separator);
						}
						data.close();
						
						finished = true;
					} catch (IOException e) {
						//e.printStackTrace();
					}
					
					if (con != null) {
						con.disconnect();
					}
				}
				
				public boolean isFinished() {
					return finished;
				}
			}

			StringBuffer buf = new StringBuffer(4000);
			URLConnectionThread t = new URLConnectionThread(url, buf);

			t.start();
			try {
				t.join(15000);
			} catch (InterruptedException e) {
				//e.printStackTrace();
			}
			t.interrupt();

			if (t.isFinished()) {
				if (file.getParentFile() != null) {
					file.getParentFile().mkdirs();
				}
				
				FileWriter w = new FileWriter(file);
				w.write(buf.toString());
				w.close();
			} else { // empty
			}
	}


	/**
	 * Copies an internet resource.
	 * 
	 * @param urlName path to the original resource.
	 * @param fileName name of the destination file.
	 * @throws IOException if any file system error occurs.
	 */
	public static void copyResourceToFile(
		String urlName,
		String fileName,
		final boolean follow)
		throws IOException {

		URL url = new URL(urlName);
		File file = new File(fileName);
		URLUtils.copyResourceToFile(url, file, follow);
	}

}
